/************************************************************************************
*
*  Project Name:	CTBRegisterInit
*     File Name:	CTBRegisterInit.c
*        Author:	Glenn L. Austin
*          Date:	February 15, 1989
*
*   Description:	Registers Serial Cards with the Macintosh Communications Toolbox.
*
*************************************************************************************
*
*	Revision History:
*		2/15/89 - Original version by Glenn L. Austin
*		2/21/89	- Added active slot display to icon
*
************************************************************************************/

#include <Types.h>
#include <Memory.h>
#include <Slots.h>
#include <Devices.h>
#include <Resources.h>
#include <ToolUtils.h>
#include <OSUtils.h>

/* include the stuff for Comm Toolbox */
#include <CTBUtils.h>
#include <CRMIntf.h>
#include <CRMSerialIntf.h>

/* what version of SysEnvirons record do we want? */
#define sysEnvironsVersion		1

/* define a shortcut to the machine information from SysEnvirons */
#define	theMac theEnviron.machineType

/* define Integer and Longint for simplification */
#define Integer		short
#define Longint		long

/* define an icon list type for later */
typedef Longint iconList[64];

/* define the Slot Param record */
typedef struct {
	Integer	boardID;
	char	vendorUse[6];		/* the bytes for vendor usage */
} SPRAMRecord, *SPRAMRecPtr;

/* define the info for the string list resource */
#define	STRLIST		1200
#define	SLOTNAME	1
#define CARDNAME	2
#define	SLOTNAME	3
#define	PORTNAME	4
#define	INNAME		5
#define OUTNAME		6
#define NUMPORTS	7

/* define which character is the driver port char */
#define DRVRCHAR	2

typedef struct {		/* QuickDraw globals */
	char private[76];
	long randSeed;
	BitMap screenBits;
	Cursor arrow;
	Pattern dkGray;
	Pattern ltGray;
	Pattern gray;
	Pattern black;
	Pattern white;
	GrafPtr thePort;
} qdGlobals;

void	RegisterCard(SpBlock *slotRec);
void	RegisterSerial(SpBlock *slotRec, Handle masterIcon, char *masterName, long numPorts);
void	DrawBootIcon(Handle theIcon);
void	FillSlot(Integer slot, Integer ports, Boolean blacken);
void	EraseSlots(void);

pascal void RegCTBCard()
{
	SysEnvRec	theEnviron;
	char		iconDrawn = false;

	(void) SysEnvirons(sysEnvironsVersion, &theEnviron);
	
/* If the Mac is known not to support the slot manager, then exit */
	if ((theMac == env512KE) || (theMac == envMacPlus) || (theMac == envSE) ||
		(theMac == envMachUnknown))
	{
		Handle	theIcon;
		
		theIcon = Get1NamedResource('ICN#',"\pNo CTBCard");
		DrawBootIcon(theIcon);
		ReleaseResource(theIcon);
		return;
	}

/* The Mac is assumed to support the slot manager */
	{
		SpBlock		slotRec;
		char		i;
		OSErr		err;
		char		*s;
		Str255		theCardName;
		int			nameLen;
		
		GetIndString(theCardName, STRLIST, SLOTNAME);
		
/* change into a C string */
		p2cstr(theCardName);
		nameLen = strlen(theCardName);

/* initialize the Comm Resource Mgr */

		InitCRM();
		
/* find the card(s)! */
		for (i = 0x9; i <= 0xE; i++)
		{
			slotRec.spSlot = i;
			slotRec.spID = 0;
			slotRec.spExtDev = 0;
			if (noErr == (err = SNextSRsrc(&slotRec)))
			{
/* the slot is not empty... */
				slotRec.spID = 2;
				if (noErr == (err = SGetCString(&slotRec)))
				{
/* the card has a string... */
					s = (char *) slotRec.spResult;
					if (strncmp(s,theCardName,nameLen) == 0)
					{
/* found a serial card! */
						if (!iconDrawn)
						{
							Handle	theIcon;
							
							iconDrawn = true;
							theIcon = Get1NamedResource('ICN#',"\pCTBCard");
							DrawBootIcon(theIcon);
							ReleaseResource(theIcon);
							EraseSlots();
						}
						RegisterCard(&slotRec);
					}
				}
				i = slotRec.spSlot;
			}
			else
				break;
		}
	}
/* if no cards were found, then draw the icon which shows that! */
	if (!iconDrawn)
	{
		Handle	theIcon;
		
		theIcon = Get1NamedResource('ICN#',"\pNo CTBCard");
		DrawBootIcon(theIcon);
		ReleaseResource(theIcon);
	}
}

/* Register a serial card! */
void RegisterCard(SpBlock *slotRec)
{
	Integer	slotNbr = (slotRec->spSlot) - 8;
	Handle	theIcon;
	Str255	theName;
	long	numPorts;
	
	theName[0] = 0;				/* to get rid of the warning */
	theIcon = Get1NamedResource('ICN#',"\pCTBCard");
	GetIndString(theName, STRLIST, CARDNAME);		/* get the base name of the card! */
	
	{												/* get the number of ports! */
		Str255	s;
		
		GetIndString(s, STRLIST, NUMPORTS);
		StringToNum(s, &numPorts);
	}
	
	FillSlot(slotNbr, numPorts, true);
	
	{
		int			theSlot, thePort;		/* for setting the icon bits */
		iconList	*theIconList = (iconList*) *theIcon;
		int			port, k;
		
		for (port = 0; port < numPorts; port++)
		{
			thePort = 19 + port * 3;		/* the longint */
			theSlot = 27 - (((slotRec->spSlot) - 9) * 5);	/* the bit number */
			
			{
				int		j;
				for (j = 0; j <= 1; j++)
					BitSet((Ptr) theIconList, (thePort) * 32 + (theSlot+j));
			}
		}
		k = 0;
		for (port = numPorts; port < 4; port++)
		{
			thePort = 17 + port * 3;		/* the longint */
			theSlot = 27 - (((slotRec->spSlot) - 9) * 5);	/* the bit number */
			
			{
				int i;
				for (i = 0; i <= 2; i++)
				{
					BitSet((Ptr) theIconList, (thePort+i) * 32 + theSlot + k);
					k = 1 - k;
				}
			}
		}
	}
	
	RegisterSerial(slotRec, theIcon, theName, numPorts);
	ReleaseResource(theIcon);
}

/* append a pascal string to the end of another pascal string and return	*/
/* a pointer to the string													*/
void pcat(char *targ, const char *src)
{
	int		i;
	int		j = targ[0];
	
	for (i = 1; i <= src[0]; i++)
		targ[i + j] = src[i];
	targ[0] += src[0];
}

void RegisterSerial(SpBlock *slotRec, Handle masterIcon, char *masterName, long numPorts)
{
	SPRAMRecord		spRec;
	OSErr			err;
	Str255			inName, outName, slotStr, portStr, tempName;
	CRMRecPtr		theCRMRec;
	CRMSerialPtr	theSerial;
	char			i;
	int				theSlot, thePort;		/* for setting the icon bits */
	iconList		*theIconList;
	
	HLock(masterIcon);						/* lock the icon down */
	
	inName[0] = outName[0] = slotStr[0] = portStr[0] = 0;	/* to get rid of the warning */
	GetIndString(inName, STRLIST, INNAME);
	GetIndString(outName, STRLIST, OUTNAME);
	GetIndString(slotStr, STRLIST, SLOTNAME);
	GetIndString(portStr, STRLIST, PORTNAME);
	
	tempName[0] = 0;
	pcat(tempName, masterName);
	pcat(tempName, slotStr);
	tempName[++tempName[0]] = (slotRec->spSlot) - 8 + '0';	/* add the slot number */
	pcat(tempName, portStr);
	tempName[++tempName[0]] = '#';		/* add a temporary port number char for */
										/* substitution of the real port number later */
	slotRec->spResult = (Longint) &spRec;
	err = SReadPRAMRec(slotRec);
	
	for (i = 0; i < numPorts; i++)
	{
		char		*p, *q;
		
		theCRMRec = (CRMRecPtr) NewPtrSys(sizeof(CRMRec));
		theSerial = (CRMSerialPtr) NewPtrSys(sizeof(CRMSerialRecord));
		
		theCRMRec->crmDeviceType = crmSerialDevice;
		theCRMRec->crmDeviceID = theCRMRec->crmStatus = theCRMRec->crmRefCon = 0;
		theCRMRec->crmAttributes = (long) theSerial;
		
		theSerial->version = curCRMSerRecVers;
		/* give the inputDriverName enough space for the length byte and the slot and port info */
		theSerial->inputDriverName = (StringHandle) NewHandleSysClear((Size) (inName[0]+3));
		HLock((Handle) (theSerial->inputDriverName));
		pcat(*(theSerial->inputDriverName),inName);
		p = (char *) *theSerial->inputDriverName;
		q = p + *p + 1;
		*q++ = (slotRec->spSlot) - 8 + '0';	/* slot number */
		*q = i + '0';
		*p += 2;
		
		/* give the outputDriverName enough space for the length byte and the slot and port info */
		theSerial->outputDriverName = (StringHandle) NewHandleSysClear((Size) (outName[0]+3));
		HLock((Handle) (theSerial->outputDriverName));
		pcat(*(theSerial->outputDriverName),outName);
		p = (char *) *theSerial->outputDriverName;
		q = p + *p + 1;
		*q++ = (slotRec->spSlot) - 8 + '0';	/* slot number */
		*q = i + '0';
		*p += 2;

		theSerial->name = (StringHandle) NewHandleSysClear((Size) (tempName[0] + 1));
		HLock((Handle) (theSerial->name));
		pcat(*(theSerial->name),tempName);
		((char *) *(theSerial->name))[tempName[0]] = i + '1';

		theSerial->deviceIcon = NewHandleSys(sizeof(iconList));
		HLock(theSerial->deviceIcon);
		BlockMove(*masterIcon, *(theSerial->deviceIcon), sizeof(iconList));
		
		thePort = 17 + i * 3;		/* the longint */
		theSlot = 27 - (((slotRec->spSlot) - 9) * 5);	/* the bit number */
		
		theIconList = (void *) *(theSerial->deviceIcon);
		{
			int		i, j;
			for (i = 0; i <= 1; i++)
				for (j = 0; j <= 1; j++)
					BitSet((Ptr) theIconList, (thePort+i) * 32 + (theSlot+j));
		}
		HUnlock((Handle) (theSerial->inputDriverName));
		HUnlock((Handle) (theSerial->outputDriverName));
		HUnlock((Handle) (theSerial->name));
		HUnlock(theSerial->deviceIcon);
		
		theSerial->ratedSpeed = 19200;
		theSerial->maxSpeed = 19200;
		
		CRMInstall((QElemPtr) theCRMRec);
	}
	HUnlock(masterIcon);
}

/* Many thanks to Paul Mercer for the ShowINIT code! */
/* However, I use MPW, so here it is in MPW 3.0 C! */

unsigned short CKSM(unsigned short i)
{
	unsigned short	highMask, highBit;
	unsigned short	val;
	
	highMask = ~((short) -1 >> 1);
	if (i & highMask)
		highBit = 1;
	else
		highBit = 0;
	val = ((i << 1) & 0xFFFE) | highBit;
	return(val ^ 0x1021);
}

void InitA5(Ptr *oldA5) = {0x2F00, 0x200D, 0x2A6F, 0x0004, 0x2A80, 0x201F, 0x584F};
void RestA5(Ptr *oldA5) = {0x2A5F, 0x2A55};

void	DrawBootIcon(Handle theIcon)
{
	short		*myH = 0x92C;		/* CurApName + 28 */
	short		*myCheck = 0x92E;	/* CurApName + 30 */
	long		*oldSig = 0xA78;	/* ApplScratch */
	short		*oldH = 0xA7E;		/* ApplScratch + 6 */
	
	short		i;
	Ptr			saveA5;
	Rect		srcRect, destRect;
	BitMap		myBitMap;
	GrafPort	myPort;
	qdGlobals	qd;
	Ptr			localA5;

	InitA5(&localA5);
	saveA5 = localA5;
	InitGraf(&qd.thePort);	
	OpenPort(&myPort);

	i = *myH;
	i = CKSM(i);
	if (i == *myCheck)
		i = *myH;
	else
		if (*oldSig == 'Paul')
			i = *oldH;
		else
			i = 8;
	destRect.bottom = myPort.portRect.bottom - 8;
	destRect.left = myPort.portRect.left + i;
	destRect.top = destRect.bottom - 32;
	destRect.right = destRect.left + 32;
	
	i += 40;
	*myH = i;
	i = CKSM(i);
	*myCheck = i;

	HLock(theIcon);
	srcRect.top = srcRect.left = 0;
	srcRect.bottom = srcRect.right = 32;
	myBitMap.rowBytes = 4;
	myBitMap.bounds = srcRect;
	myBitMap.baseAddr = *theIcon + 128;	/* Skip to mask */
	CopyBits(&myBitMap, &myPort.portBits, &srcRect, &destRect, srcBic, 0L);
	myBitMap.baseAddr = *theIcon;			/* Now draw icon */
	CopyBits(&myBitMap, &myPort.portBits, &srcRect, &destRect, srcOr, 0L);
	HUnlock(theIcon);
	
	ClosePort(&myPort);
	RestA5(&saveA5);
	*oldSig = 'Paul';
}

void EraseSlots()
{
	Integer		i;
	
	for (i = 1; i <= 6; i++)
		FillSlot(i, 4, false);
}

void FillSlot(Integer slot, Integer ports, Boolean blacken)
{
	short		*myH = 0x92C;		/* CurApName + 28 */
	short		*myCheck = 0x92E;	/* CurApName + 30 */
	long		*oldSig = 0xA78;	/* ApplScratch */
	short		*oldH = 0xA7E;		/* ApplScratch + 6 */
	
	short		i;
	Ptr			saveA5;
	Integer		theSlot;
	Rect		srcRect, destRect;
	GrafPort	myPort;
	qdGlobals	qd;
	Ptr			localA5;

	InitA5(&localA5);
	saveA5 = localA5;
	InitGraf(&qd.thePort);	
	OpenPort(&myPort);

	i = *myH;
	i = CKSM(i);
	if (i == *myCheck)
		i = *myH;
	else
		if (*oldSig == 'Paul')
			i = *oldH;
		else
			i = 8;

	i -= 40;	/* move back to my icon */
	
	destRect.bottom = myPort.portRect.bottom - 8;
	destRect.left = myPort.portRect.left + i;
	destRect.top = destRect.bottom - 32;
	destRect.right = destRect.left + 32;
	
	theSlot = 27 - ((slot - 1) * 5);	/* the bit number */
	
	srcRect.top = 17 + destRect.top;
	srcRect.bottom = 18 + ((ports >> 1) * 5) + destRect.top;
	srcRect.left = theSlot + destRect.left;
	srcRect.right = theSlot + 2 + destRect.left;
	if (blacken)
		FillRect(&srcRect, qd.black);
	else
		FillRect(&srcRect, qd.white);

	ClosePort(&myPort);
	RestA5(&saveA5);
}	